import {IncomingMessage} from 'http';
import {PATCH_CONTENT_TYPES} from '../../../common';

export const isTextMediaType = (mediaType: string) => (
	mediaType.startsWith('text/')
	|| [
		'application/json',
		'application/xml',
		'application/x-www-form-urlencoded',
		...PATCH_CONTENT_TYPES,
	].includes(mediaType)
);

export const getBody = (
	req: IncomingMessage,
) => new Promise<Buffer>((resolve, reject) => {
	let body = Buffer.from('');
	req.on('data', (chunk) => {
		body = Buffer.concat([body, chunk]);
	});
	req.on('end', async () => {
		resolve(body);
	});

	req.on('error', (err) => {
		reject(err);
	})
});

interface LinkMapEntry {
	url: string;
	params: Record<string, string>;
}

export class LinkMap extends Set<LinkMapEntry> {
	toString() {
		const entries = Array.from(this.values());

		return entries.map((e) => {
			const params = Object.entries(e.params);

			return [
				`<${e.url}>`,
				...params.map(([key, value]) => `${encodeURIComponent(key)}="${encodeURIComponent(value)}"`)
			].join(';')
		}).join(',');
	}
}
