export interface MediaType<
	Name extends string = string,
	T extends object = object,
	SerializeOpts extends unknown[] = [],
	DeserializeOpts extends unknown[] = []
> {
	name: Name;
	serialize: (object: T, ...args: SerializeOpts) => string;
	deserialize: (s: string, ...args: DeserializeOpts) => T;
}

export const FALLBACK_MEDIA_TYPE = {
	serialize: (obj: unknown) => JSON.stringify(obj),
	deserialize: (str: string) => JSON.parse(str),
	name: 'application/json' as const,
} satisfies MediaType;

export const PATCH_CONTENT_TYPES = [
	'application/merge-patch+json',
	'application/json-patch+json',
] as const;

export type PatchContentType = typeof PATCH_CONTENT_TYPES[number];

export const getAcceptPostString = (mediaTypes: Map<string, MediaType>) => Array.from(mediaTypes.keys())
	.filter((t) => !PATCH_CONTENT_TYPES.includes(t as PatchContentType))
	.join(',');
